close all
clear all

% -------------------------------------------------------------------------
% Get where this m-file is
CurrentPath = strrep(mfilename('fullpath'), mfilename, '');

% Add it to the path list
addpath(CurrentPath);

% And go there
cd(CurrentPath);

% -------------------------------------------------------------------------
% Define the name of the file we're looking for
CladeInfoFileName = 'cladogram_data';

% -------------------------------------------------------------------------
% Read in the text data; this is tab-delimited text, with four columns
% consisting of the name of the clade, the name of its parent, whether,
% when it branches off its parent it does this concurrently (at the same
% point as) other children, or sequentially (after preceding children), and
% lastly a number indicating how much of its span the clade is extinct for
% (as indicated by a dotted line on the output).
[CladeNames, ...
 CladeParent, ...
 SequentialOrConcurrentBranching, ...
 BecomesExtinctAt] = ...
    textread([CladeInfoFileName '.txt'], '%q %q %u %f');

% -------------------------------------------------------------------------
% Initialise the counter for the number of children a clade has
NumberOfChildren     = zeros(size(CladeNames));
CladeParentPosition  = zeros(size(CladeNames));
CladeGeneration      = ones(size(CladeNames));
OverallCladePosition = zeros(size(CladeNames));

% -------------------------------------------------------------------------
% Initialise a cell array to store the details of any childen each clade
% may have
CladeChildren    = cell(length(CladeNames), 1);

% -------------------------------------------------------------------------
% Now find which things are the children of what
for i = 1:length(CladeParent)
    
    % See what parent matches this clade parent, if any
    FindResult = find(strcmp(CladeParent(i), CladeNames));
    
    if ~isempty(FindResult)
        
        % We have a match so, for the child, store the position of its parent
        CladeParentPosition(i) = FindResult;
        
        % And for the parent increment the number of children it has
        NumberOfChildren(FindResult) = NumberOfChildren(FindResult) + 1;
        
        % And for the parent store the references to all the children it
        % has; first check is this is the first one, in which create an
        % output cell array
        if NumberOfChildren(FindResult) == 1
            CladeChildren(FindResult, 1) = {cell(1, 1)};
        end
        
        % And store the reference to the child in the parents reference array
        CladeChildren{FindResult, 1}(1, NumberOfChildren(FindResult)) = {i};
        
    end
    
end

% -------------------------------------------------------------------------
% And from that assign a generation to everything; we do this by, for each
% clade, finding its parent and following the links to its parents until we
% get to one without a parent (CurrentParent of zero) which is the LCA. The
% number of such links gives us the generation of the current clade.
for i = 1:length(CladeParentPosition)
    
    FoundGeneration = false;
    CurrentParent   = CladeParentPosition(i);
    
    if CurrentParent ~= 0
        
        while ~FoundGeneration

            CurrentParent      = CladeParentPosition(CurrentParent);
            CladeGeneration(i) = CladeGeneration(i) + 1;

            if CurrentParent == 0

                FoundGeneration = true;

            end

        end
    
    else
        
        FoundGeneration = true;

    end
    
end

% -------------------------------------------------------------------------
% And get the highest generation we have; useful for drawing the cladogram
HighestGeneration              = max(CladeGeneration);

NumberOfSecondGenerationClades = sum(nonzeros(CladeGeneration == 2));

% OK to here

CurrentPositionCounter = 0;

% -------------------------------------------------------------------------
% Find where to begin traversing the clade tree
for i = 1:length(CladeNames)
    
    % Find the position where the generation is 1
    if CladeGeneration(i) == 1
        
        % And go through all of its children, and their children in turn,
        % to get the order in which clades should be output
        [OverallCladePosition, ...
         CurrentPositionCounter] = ...
            ...
            Get_Child_Positions(OverallCladePosition, ...
                                CurrentPositionCounter, ...
                                i, ...
                                CladeChildren);

        % And exit the loop; there should only be one 1st generation case
        break;
        
    end
    
end

% -------------------------------------------------------------------------
% Initialise the offset to be applied to the start of each clade
CladeStartPositionShift = zeros(size(CladeNames));

% -------------------------------------------------------------------------

% Get the clade in generation order, so that parents always exist for
% children. This is also predicated on each child having only 1 parent.
[SortedGeneration, GenerationIndices] = sort(CladeGeneration);

% Initialise things
CladeX              = zeros(size(CladeNames));
CladeXIncrement     = 1;

% Go through all clades
for k = 1:length(CladeNames)
    
    % But in order of generation, so that parents always exist for children
    i = GenerationIndices(k);

    SequentialOffset = 1;

    % If this clade has some children work out their X coordinates
    if ~isempty(CladeChildren{i})
        
        for j = 1:length(CladeChildren{i})

            % The initial X for all children is the X of their parent, plus
            % one
            CladeX(CladeChildren{i}{j}) = CladeX(i) + 1;
            
            % But if some children branch sequentially we may need to add
            % to that
            if SequentialOrConcurrentBranching(CladeChildren{i}{j}) == 1
                
                % So add the offset
                CladeX(CladeChildren{i}{j}) = CladeX(CladeChildren{i}{j}) + SequentialOffset;
                
                % And increment it so each sequential child is offset to
                % the right by 1
                SequentialOffset            = SequentialOffset            + 1;
                
            end

        end

    end
   
end

% And save the overall endpoint for working out the right hand end of each
% clade lifeline
OverallEndPointShift = max(CladeX) + 3;

% -------------------------------------------------------------------------
% Work out whether children are first child and then synch offset
% multiplied by that so first children do not have their position 
% offset. For the rest once that's been done, if it's the length 
% (max generation) wants to have the sum of the remaining offsets added 
% to it to extend the lines to the right. When going through to plot the 
% lines the start of each line wants to be offset by the synch offset 
% flag for that line; should work!

% -------------------------------------------------------------------------
% Work out the coordinates of the lines for the different clades
% Order the clades along the y axis so they are in the correct sequence,
% taking into account the parents and children

CladeY                 = zeros(size(CladeNames));
MaxNumberOfChildren    = max(NumberOfChildren) + 1;

ThisStartPositionIndex = zeros(size(CladeNames));
StartPositionShifts    = zeros(size(CladeNames));
CladeYMultiplier       = zeros(size(CladeNames));
CladeThisAppliesTo     = zeros(size(CladeNames));

CladeYIncrement        = MaxNumberOfChildren .^ (HighestGeneration - CladeGeneration + 1);
    % We can just do all of these at once as they're independant of position

% And get the clade start positions sorted in order, from largest to
% smallest
[SortedStartPositions, StartPositionIndices] = sort(CladeX, 'descend');

% Go through each clade in turn; here we work out the Y positions of it
% children; this means the parent clade Y value is always zero (which is
% what we want anyway!).
for k = 1:length(CladeNames)
    
    % But in order of generation, so that parents always exist for children
    i = GenerationIndices(k);

    % If this clade has some children work out their Y coordinates
    if ~isempty(CladeChildren{i})
        
        % Get rid of old data (if any)
        clear ThisStartPositionIndex;
        clear StartPositionShifts;
        clear CladeYMultiplier;
        clear CladeThisAppliesTo;
    
        % First get the positions of the children of this clade, and the
        % start position shifts corresponding to them
        for j = 1:length(CladeChildren{i})

            ThisStartPositionIndex(j) = find(CladeChildren{i}{j} == StartPositionIndices);
            StartPositionShifts(j)    = SortedStartPositions(ThisStartPositionIndex(j));

        end

        % Now sort the extracted start position shifts with the largest
        % first
        [SortedShifts, ShiftIndices] = sort(StartPositionShifts, 'descend');

        % Now go through these start position shifts in turn
        for j = 1:length(StartPositionShifts)
            
            % Get the multiplier for this child
            CladeYMultiplier(j) = j;
            
            % And find the clade this applies to; this is basically done by
            % looking back through the pointers to the original data
            CladeThisAppliesTo(j) = CladeChildren{i}{ShiftIndices(j)};
            
            % And from that calculate the clade Y, taking into account the
            % Y position of the clade parent.
            CladeY(CladeThisAppliesTo(j)) = (CladeYIncrement(CladeThisAppliesTo(j)) * ...
                                             CladeYMultiplier(j)) + ...
                                             CladeY(i);
                                         
        end

    end
   
end

% -------------------------------------------------------------------------
% Now normalise everything to be equally spaced; we do this by finding the
% smallest values in turn and storing the loop counter for that points
% index as the new y value; this is _not_ the same as just sorting it.

TempCladeY       = CladeY;
CladeYNormalised = NaN(size(CladeNames));

for i = 1:length(CladeY)

    [MinVal, MinIndex]         = min(TempCladeY);
    CladeYNormalised(MinIndex) = i - 1;
        % Subtract 1 as the y value of the minimum is 0 not 1
    TempCladeY(MinIndex)       = NaN;

end

% -------------------------------------------------------------------------
% Now work out the actual positions of the cladogram line for each clade,
% taking into account all of the above 
CladeExtantXPositions  = NaN(length(CladeNames), 3);
CladeExtantYPositions  = NaN(length(CladeNames), 3);
CladeExtinctYPositions = NaN(length(CladeNames), 2);
CladeExtinctXPositions = NaN(length(CladeNames), 2);

for i = 1:length(CladeNames)
    
    % Calculate the bit of the clade line before it went extinct
    if CladeGeneration(i) == 1
        
        StartY                       = 0;
        CladeExtantYPositions(i, :)  = [0 0 0];
        CladeExtinctYPositions(i, :) = [0 0];
        
    else
        
        StartY                       = CladeYNormalised(CladeParentPosition(i));
        
        CladeExtantYPositions(i, :)  = [-StartY ...
                                        -CladeYNormalised(i) ...
                                        -CladeYNormalised(i)];
        CladeExtinctYPositions(i, :) = [-CladeYNormalised(i) ...
                                        -CladeYNormalised(i)];
                                                      
    end
    
    CladeExtantXPositions(i, 1) = CladeX(i);
    
    CladeExtantXPositions(i, 2) = CladeX(i) + 0.5;
    
    ExtantX       = OverallEndPointShift - ...
                    ((OverallEndPointShift - CladeExtantXPositions(i, 2)) * BecomesExtinctAt(i));
                   
    CladeExtantXPositions(i, 3) = ExtantX;
    
    % And do the bit after extinction, if any
    CladeExtinctXPositions(i, :) = ...
        [ExtantX OverallEndPointShift];
    
end

% -------------------------------------------------------------------------
% Plot the cladogram
plot_the_cladogram( ...
        CladeNames, ...
        CladeExtantXPositions, ...
        CladeExtantYPositions, ...
        CladeExtinctXPositions, ...
        CladeExtinctYPositions, ...
        strrep(CladeInfoFileName, '.txt', '') ...
        );
    
% -------------------------------------------------------------------------
% And we're done...
% -------------------------------------------------------------------------
